/* Ejemplo de manejo de archivos en C
 * printself.c: programa que imprime por pantalla su propio código fuente
 * Clase 08 - Sábado 13/6/15
 * Autor: Leandro Kollenberger
 * Compilar con: gcc printfself.c -o printself -Wall
 * Ejecutar con: ./printself
 */

#include <stdio.h> // Necesario para fopen, fclose, fgets, fprintf, fputs, y FILE *.
#include <string.h> // Necesario para memset.

int main(void) {
	FILE *fp; // Pido un descriptor de archivo, lo llamo "fp"
	char buffer[512]; // Pido un vector de caracteres, donde voy a cargar las líneas que lea
	
	fp = fopen("./printself.c", "r"); // Abro el archivo "printself.c", ubicado en la
									  // carpeta actual, en modo de lectura
	
	if(fp == NULL) {	// Ya que el descriptor de archivo es realmente un puntero, 
						// fopen nos devuelve un NULL en caso de error.
						
		// En ese caso, imprimimos un error por la salida estándar de errores (stderr).
		fprintf(stderr, "Error abriendo archivo \"printself.c\".\n"); 
		
	} else {	// En caso exitoso, seguimos con el programa...
	
		while(!feof(fp)) {	// feof() devuelve un 1 en caso de que sea el final del archivo, 
							// así que lo sigo leyendo mientras devuelva un cero.
			memset(buffer, 0, 512); // Limpio el array entero llenándolo de ceros con memset()
			
			fgets(buffer, 511, fp); // Leo una línea, o hasta 511 bytes, lo que ocurra primero.
									// El buffer es de 512 bytes, pero siempre hay que dejar un 
									// '\0' al final del string.
									
			fputs(buffer, stdout);  // Con fputs() escribo el string que leí del archivo a la 
									// salida estándar de pantalla.
		}
		
		fclose(fp); // Finalizo el programa, cerrando el archivo.
	}
	
	return 0;
}

/* Errores comúnes al laburar con archivos:
 *   - Olvidar comprobar el resultado de fopen(). Hay que comprobar siempre que devuelva algo
 *     distinto de NULL. En caso de que haya un error, al intentar leer o escribir de ese 
 *     descriptor de archivo, tendremos un Segmentation Fault.
 *   - Cerrar con fclose() un FILE * que apunte a NULL: en caso de que fopen() tenga un error
 *     (y el descriptor apunte a NULL) e intentar cerrarlo con fclose(), tirará un Segmentation Fault.
 *   - Olvidar cerrar el archivo: Es la menos "fatal" de todas, pero de todos modos es incorrecta,
 *     ya que causaría que nuestro programa "pierda" memoria (ya lo veremos con más detalle cuando
 *     veamos manejo dinámico de memoria).
 */
