/* Ejemplo de manejo de archivos en C
 * queridodiario.c: lee líneas por pantalla y las escribe a un archivo
 *                  de diario personal, agregándole la fecha y hora.
 *                  Termina cuando el usuario ingresa un "enter" dos veces
 *                  seguidas.
 * Clase 08 - Sábado 13/6/15
 * Autor: Leandro Kollenberger
 * Compilar con: gcc queridodiario.c -o queridodiario -Wall
 * Ejecutar con: ./queridodiario
 */

#include <stdio.h>
#include <time.h>
#include <string.h>

int main(void) {
    FILE *archivo;
    time_t hora;
    char hora_s[100];
    char entrada[512];

    // Cargo la hora actual en el time_t. time_t es un número entero que
    // contiene la hora en segundos, desde el Epoch (googlear "Unix time")
    // para más detalles).
    hora = time(NULL);
    // La paso a string. ctime() me devuelve un char * con la dirección de
    // comienzo del string con la hora, la cual debo copiar a un vector mío.
    // Ya que ctime() recibe un puntero a time_t, debo pasarle la dirección
    // de memoria de mi time_t, con la hora y fecha que deseo convertir.
    strcpy(hora_s, ctime(&hora));

    // Abro el archivo "diario.txt", ubicado en el directorio actual,
    // en modo "append", es decir, para escribir al final del archivo.
    archivo = fopen("./diario.txt", "a");

    // Compruebo si pude abrir exitosamente el archivo.
    if( archivo != NULL ) {
        // En caso de que lo haya podido abrir, imprimo la fecha y hora
        // que armé anteriormente.
        fprintf(archivo, "%s\n", hora_s);

        // Comienzo a leer de teclado...
        do {
            // Lleno el string de ceros para limpiarlo.
            memset(entrada, 0, 512);
            // Leo una línea de pantalla, cuidando de no pasarme de los
            // 511 bytes disponibles.
            fgets(entrada, 511, stdin);
            // "Imprimo" el string que leí de pantalla en el archivo
            // con fprintf(), dejando una tabulación para que queden
            // las entradas del día de manera ordenada.
            fprintf(archivo, "\t%s", entrada);
        // Esto lo sigo haciendo hasta que el primer caracter de la línea leída
        // sea un '\n' (es decir, el usuario únicamente ingresó un enter indicando
        // que terminó el ingreso de datos) o que sea un '\0' (en caso de que no se
        // haya logrado leer nada o el string quedó vacío).
        } while( entrada[0] != '\0' && entrada[0] != '\n' );

        // Cierro el archivo, y chequeo el valor de retorno de fclose().
        // Esta función me devuelve un 1 en caso de error.
        if( !fclose(archivo) )
            printf("Archivo guardado con exito.\n");
        else
            fprintf(stderr, "Error cerrando archivo!\n");
    // No pude abrir el archivo, imprimo error por la salida estándar de error.
    } else
        fprintf(stderr, "Error abriendo archivo!\n");

    return 0;
}
